@@ -3,6 +3,7 @@
 
 import logging
 import os.path
+import sys
 from abc import ABC, abstractmethod
 from hashlib import sha256
 from typing import BinaryIO, Callable, Generic, Optional, Sequence, TypeVar
@@ -10,6 +11,7 @@ from typing import BinaryIO, Callable, Generic, Optional, Sequence, TypeVar
 import click
 from cryptography import x509
 from cryptography.hazmat.primitives.asymmetric import ec
+from nitrokey import trussed
 from nitrokey.trussed import (
     FirmwareContainer,
     Model,
@@ -23,12 +25,14 @@ from nitrokey.trussed import (
 )
 from nitrokey.trussed.admin_app import BootMode, InitStatus, Status
 from nitrokey.trussed.provisioner_app import ProvisionerApp
+from nitrokey.trussed.updates import Warning
 from nitrokey.updates import OverwriteError
 
 from pynitrokey.cli.exceptions import CliException
 from pynitrokey.helpers import (
     DownloadProgressBar,
     Retries,
+    Table,
     local_critical,
     local_print,
     require_windows_admin,
@@ -60,11 +64,11 @@ class Context(ABC, Generic[Bootloader, Device]):
     @abstractmethod
     def test_cases(self) -> Sequence[TestCase]: ...
 
-    @abstractmethod
-    def open(self, path: str) -> Optional[TrussedBase]: ...
+    def open(self, path: str) -> Optional[TrussedBase]:
+        return trussed.open(path, model=self.model)
 
-    @abstractmethod
-    def list_all(self) -> Sequence[TrussedBase]: ...
+    def list_all(self) -> Sequence[TrussedBase]:
+        return trussed.list(model=self.model)
 
     def list(self) -> Sequence[TrussedBase]:
         if self.path:
@@ -77,20 +81,20 @@ class Context(ABC, Generic[Bootloader, Device]):
             return self.list_all()
 
     def connect(self) -> TrussedBase:
-        return self._select_unique(self.model.name, self.list())
+        return self._select_unique(str(self.model), self.list())
 
     def connect_device(self) -> Device:
         devices = [
             device for device in self.list() if isinstance(device, self.device_type)
         ]
-        return self._select_unique(self.model.name, devices)
+        return self._select_unique(str(self.model), devices)
 
     def await_device(
         self,
         retries: Optional[int] = None,
         callback: Optional[Callable[[int, int], None]] = None,
     ) -> Device:
-        return self._await(self.model.name, self.device_type, retries, callback)
+        return self._await(str(self.model), self.device_type, retries, callback)
 
     def await_bootloader(
         self,
@@ -98,7 +102,7 @@ class Context(ABC, Generic[Bootloader, Device]):
         callback: Optional[Callable[[int, int], None]] = None,
     ) -> Bootloader:
         return self._await(
-            f"{self.model.name} bootloader", self.bootloader_type, retries, callback
+            f"{self.model} bootloader", self.bootloader_type, retries, callback
         )
 
     def _select_unique(self, name: str, devices: Sequence[T]) -> T:
@@ -145,14 +149,21 @@ def prepare_group() -> None:
     require_windows_admin()
 
 
-def add_commands(group: click.Group) -> None:
+def add_commands(group: click.Group, *, has_app_reset: bool = True) -> None:
     group.add_command(fetch_update)
     group.add_command(list)
+    group.add_command(list_config_fields)
+    group.add_command(get_config)
+    group.add_command(set_config)
+    group.add_command(factory_reset)
+    if has_app_reset:
+        group.add_command(factory_reset_app)
     group.add_command(provision)
     group.add_command(reboot)
     group.add_command(rng)
     group.add_command(status)
     group.add_command(test)
+    group.add_command(update)
     group.add_command(validate_update)
     group.add_command(version)
 
@@ -223,7 +234,7 @@ def list(ctx: Context[Bootloader, Device]) -> None:
 
 
 def _list(ctx: Context[Bootloader, Device]) -> None:
-    local_print(f":: '{ctx.model.name}' keys")
+    local_print(f":: '{ctx.model}' keys")
     for device in ctx.list_all():
         with device as device:
             uuid = device.uuid()
@@ -233,6 +244,186 @@ def _list(ctx: Context[Bootloader, Device]) -> None:
                 local_print(f"{device.path}: {device.name}")
 
 
+@click.command()
+@click.pass_obj
+def list_config_fields(ctx: Context[Bootloader, Device]) -> None:
+    """
+    List all supported config fields.
+
+    This commands lists all config fields that can be accessed with get-config
+    and set-config as well as their type. The possible types are Bool ("true"
+    or "false") and U8 (an integer between 0 and 255).
+
+    The available config fields depend on the firmware version of the device.
+    """
+    with ctx.connect_device() as device:
+        fields = device.admin.list_available_fields()
+
+        table = Table(["config field", "type"])
+        for field in fields:
+            table.add_row(
+                [
+                    field.name,
+                    field.ty,
+                ]
+            )
+        local_print(table)
+
+
+@click.command()
+@click.pass_obj
+@click.argument("key")
+def get_config(ctx: Context[Bootloader, Device], key: str) -> None:
+    """Query a config value."""
+    with ctx.connect_device() as device:
+        value = device.admin.get_config(key)
+        print(value)
+
+
+@click.command()
+@click.pass_obj
+@click.argument("key")
+@click.argument("value")
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    default=False,
+    help="Set the config value even if it is not known to pynitrokey",
+)
+@click.option(
+    "--dry-run",
+    is_flag=True,
+    default=False,
+    help="Perform all checks but don’t execute the configuration change",
+)
+def set_config(
+    ctx: Context[Bootloader, Device], key: str, value: str, force: bool, dry_run: bool
+) -> None:
+    """
+    Set a config value.
+
+    Per default, this command can only be used with configuration values that
+    are known to pynitrokey.  Changing some configuration values can have side
+    effects.  For these values, a summary of the effects of the change and a
+    confirmation prompt will be printed.
+
+    If you use the --force/-f flag, you can also set configuration values that
+    are not known to pynitrokey.  This may have unexpected side effects, for
+    example resetting an application.  It is only intended for development and
+    testing.
+
+    To see the information about a config value without actually performing the
+    change, use the --dry-run flag.
+    """
+
+    with ctx.connect_device() as device:
+        config_fields = device.admin.list_available_fields()
+
+        field_metadata = None
+        for field in config_fields:
+            if field.name == key:
+                field_metadata = field
+
+        if field_metadata is None:
+            print(
+                "Changing configuration values can have unexpected side effects, including data loss.",
+                file=sys.stderr,
+            )
+            print(
+                "This should only be used for development and testing.",
+                file=sys.stderr,
+            )
+            if not force:
+                raise CliException(
+                    "Unknown config values can only be set if the --force/-f flag is set.  Aborting.",
+                    support_hint=False,
+                )
+
+        if (
+            not force
+            and field_metadata is not None
+            and not field_metadata.ty.is_valid(value)
+        ):
+            raise CliException(
+                f"Invalid config value for {field}: expected {field_metadata.ty}, got `{value}`. Unknown config values can only be set if the --force/-f flag is set.  Aborting.",
+                support_hint=False,
+            )
+
+        if key == "opcard.use_se050_backend":
+            print(
+                "This configuration values determines whether the OpenPGP Card "
+                "application uses a software implementation or the secure element.",
+                file=sys.stderr,
+            )
+            print(
+                "Changing this configuration value will cause a factory reset of "
+                "the OpenPGP card application and destroy all OpenPGP keys and "
+                "user data currently stored on the device.",
+                file=sys.stderr,
+            )
+        elif field_metadata is not None and field_metadata.destructive:
+            print(
+                "This configuration value may delete data on your device",
+                file=sys.stderr,
+            )
+
+        if field_metadata is not None and field_metadata.destructive:
+            click.confirm("Do you want to continue anyway?", abort=True)
+
+        if dry_run:
+            print("Stopping dry run.", file=sys.stderr)
+            raise click.Abort()
+
+        if field_metadata is not None and field_metadata.requires_touch_confirmation:
+            print(
+                "Press the touch button to confirm the configuration change.",
+                file=sys.stderr,
+            )
+
+        device.admin.set_config(key, value)
+
+        if field_metadata is not None and field_metadata.requires_reboot:
+            print("Rebooting device to apply config change.")
+            device.reboot()
+
+        print(f"Updated configuration {key}.")
+
+
+@click.command()
+@click.pass_obj
+def factory_reset(ctx: Context[Bootloader, Device]) -> None:
+    """Factory reset all functionality of the device"""
+
+    with ctx.connect_device() as device:
+        local_print("Please touch the device to confirm the operation", file=sys.stderr)
+        if not device.admin.factory_reset():
+            local_critical(
+                "Factory reset is not supported by the firmware version on the device",
+                support_hint=False,
+            )
+
+
+# opcard is the only application that supports factory reset at the moment
+# see the reset_client_id implementations in components/apps/src/lib.rs in nitrokey-3-firmware
+APPLICATIONS_CHOICE = click.Choice(["opcard"])
+
+
+@click.command()
+@click.pass_obj
+@click.argument("application", type=APPLICATIONS_CHOICE, required=True)
+def factory_reset_app(ctx: Context[Bootloader, Device], application: str) -> None:
+    """Factory reset all functionality of an application"""
+
+    with ctx.connect_device() as device:
+        local_print("Please touch the device to confirm the operation", file=sys.stderr)
+        if not device.admin.factory_reset_app(application):
+            local_critical(
+                "Application Factory reset is not supported by the firmware version on the device",
+                support_hint=False,
+            )
+
+
 @click.group(hidden=True)
 def provision() -> None:
     """
@@ -506,9 +697,9 @@ def test(
 
     if len(devices) == 0:
         log_devices()
-        raise CliException(f"No connected {ctx.model.name} devices found")
+        raise CliException(f"No connected {ctx.model} devices found")
 
-    local_print(f"Found {len(devices)} {ctx.model.name} device(s):")
+    local_print(f"Found {len(devices)} {ctx.model} device(s):")
     for device in devices:
         local_print(f"- {device.name} at {device.path}")
 
@@ -537,6 +728,64 @@ def test(
         raise CliException(f"Test failed for {failure} device(s)")
 
 
+@click.command()
+@click.argument("image", type=click.Path(exists=True, dir_okay=False), required=False)
+@click.option(
+    "--version",
+    help="Set the firmware version to update to (default: latest stable)",
+)
+@click.option(
+    "--ignore-pynitrokey-version",
+    default=False,
+    is_flag=True,
+    help="Allow updates with an outdated pynitrokey version (dangerous)",
+)
+@click.option(
+    "--ignore-warning",
+    help="Ignore the warning(s) with the given ID(s) during the update (dangerous)",
+    type=click.Choice([w.value for w in Warning]),
+    multiple=True,
+)
+@click.option(
+    "--confirm",
+    default=False,
+    is_flag=True,
+    help="Confirm all questions to allow running non-interactively",
+)
+@click.pass_obj
+def update(
+    ctx: Context[Bootloader, Device],
+    image: Optional[str],
+    version: Optional[str],
+    ignore_warning: Sequence[str],
+    ignore_pynitrokey_version: bool,
+    confirm: bool,
+) -> None:
+    """
+    Update the firmware of the device using the given image.
+
+    This command requires that exactly one Nitrokey in bootloader or firmware mode is connected.
+    The user is asked to confirm the operation before the update is started.  If the --confirm
+    option is provided, this is the confirmation.  This option may be used to automate an update.
+    The Nitrokey may not be removed during the update.  Also, additional Nitrokey devices may
+    not be connected during the update.
+
+    If no firmware image is given, the latest firmware release is downloaded automatically.  If
+    the --version option is set, the given version is downloaded instead.
+
+    If the connected Nitrokey device is in firmware mode, the user is prompted to touch the
+    device’s button to confirm rebooting to bootloader mode.
+    """
+
+    from .update import update as exec_update
+
+    ignore_warnings = frozenset([Warning.from_str(s) for s in ignore_warning])
+    update_to_version, status = exec_update(
+        ctx, image, version, ignore_pynitrokey_version, ignore_warnings, confirm
+    )
+    print_status(update_to_version, status)
+
+
 @click.command()
 @click.argument("image", type=click.Path(exists=True, dir_okay=False))
 @click.pass_obj
